@@ -2,32 +2,31 @@
 # -*- coding: utf-8 -*-
 #
 # ***** BEGIN LICENSE BLOCK *****
-# Copyright (C) 2012-2014, Hayaki Saito 
-# 
-# Permission is hereby granted, free of charge, to any person obtaining a 
-# copy of this software and associated documentation files (the "Software"), 
-# to deal in the Software without restriction, including without limitation 
-# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
-# and/or sell copies of the Software, and to permit persons to whom the 
-# Software is furnished to do so, subject to the following conditions: 
-# 
-# The above copyright notice and this permission notice shall be included in 
-# all copies or substantial portions of the Software. 
-# 
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
-# DEALINGS IN THE SOFTWARE. 
-# 
+# Copyright (C) 2012-2014, Hayaki Saito
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 # ***** END LICENSE BLOCK *****
 
-__author__  = "Hayaki Saito (user@zuse.jp)"
-__version__ = "0.2.9"
+__author__ = "Hayaki Saito (user@zuse.jp)"
+__version__ = "0.2.10"
 __license__ = "MIT"
-signature   = '2aaf89c556a5779adf2d6749de196cad'
+signature = 'b87c36758a4c3d666c74490b383f483b'
 
 import sys
 import os
@@ -45,11 +44,13 @@ import logging
 _BUFFER_SIZE = 8192
 _ESC_TIMEOUT = 0.5  # sec
 
+
 ###############################################################################
 #
 # Exceptions
 #
 class NotHandledException(Exception):
+
     ''' thrown when an unknown seqnence is detected '''
 
     def __init__(self, value):
@@ -70,6 +71,7 @@ class NotHandledException(Exception):
 
 
 class ParseException(Exception):
+
     ''' thrown when a parse error is detected '''
 
     def __init__(self, value):
@@ -100,6 +102,7 @@ class ParseException(Exception):
 # - PTY
 #
 class EventObserver:
+
     ''' adapt to event driven ECMA-35/48 parser model '''
 
     def handle_start(self, context):
@@ -114,23 +117,30 @@ class EventObserver:
     def handle_esc(self, context, prefix, final):
         raise NotImplementedError("EventObserver::handle_esc")
 
+    def handle_ss2(self, context, final):
+        raise NotImplementedError("EventObserver::handle_ss2")
+
+    def handle_ss3(self, context, final):
+        raise NotImplementedError("EventObserver::handle_ss3")
+
     def handle_control_string(self, context, prefix, value):
         raise NotImplementedError("EventObserver::handle_control_string")
 
     def handle_char(self, context, c):
         raise NotImplementedError("EventObserver::handle_char")
 
-    def handle_draw(self, context):
-        raise NotImplementedError("EventObserver::handle_draw")
-
     def handle_invalid(self, context, seq):
         raise NotImplementedError("EventObserver::handle_invalid")
 
+    def handle_draw(self, context):
+        raise NotImplementedError("EventObserver::handle_draw")
+
     def handle_resize(self, context, row, col):
         raise NotImplementedError("EventObserver::handle_resize")
 
 
 class Scanner:
+
     ''' forward input iterator '''
 
     def __iter__(self):
@@ -143,7 +153,9 @@ class Scanner:
     def continuous_assign(self, value, termenc):
         raise NotImplementedError("Scanner::continuous_assign")
 
+
 class OutputStream:
+
     ''' abstruct TTY output stream '''
 
     def write(self, c):
@@ -154,6 +166,7 @@ class OutputStream:
 
 
 class EventDispatcher:
+
     ''' Dispatch interface of terminal sequence event oriented parser '''
 
     def dispatch_esc(self, prefix, final):
@@ -170,6 +183,7 @@ class EventDispatcher:
 
 
 class Parser:
+
     ''' abstruct Parser '''
 
     def parse(self, context):
@@ -177,6 +191,7 @@ class Parser:
 
 
 class PTY:
+
     ''' abstruct PTY device '''
 
     def fitsize(self):
@@ -206,6 +221,7 @@ class PTY:
 # Simple Parser implementation
 #
 class SimpleParser(Parser):
+
     ''' simple parser, don't parse ESC/CSI/string seqneces '''
 
     class _MockContext:
@@ -265,6 +281,7 @@ class _MockHandler:
 
 
 class DefaultParser(Parser):
+
     ''' parse ESC/CSI/string seqneces '''
 
     def __init__(self):
@@ -573,12 +590,12 @@ class DefaultParser(Parser):
         self.__state = state
 
 
-
 ###############################################################################
 #
 # Scanner implementation
 #
 class DefaultScanner(Scanner):
+
     ''' scan input stream and iterate UCS code points '''
 
     def __init__(self, ucs4=True, termenc=None):
@@ -641,13 +658,14 @@ class DefaultScanner(Scanner):
                 yield ord(x)
 
 
-
 ###############################################################################
 #
 # Handler implementation
 #
 class DefaultHandler(EventObserver):
+
     ''' default handler, pass through all ESC/CSI/string seqnceses '''
+
     def __init__(self):
         pass
 
@@ -729,6 +747,16 @@ class FilterMultiplexer(EventObserver):
         handled_rhs = self.__rhs.handle_esc(context, intermediate, final)
         return handled_lhs and handled_rhs
 
+    def handle_ss2(self, context, final):
+        handled_lhs = self.__lhs.handle_ss2(context, final)
+        handled_rhs = self.__rhs.handle_ss2(context, final)
+        return handled_lhs and handled_rhs
+
+    def handle_ss3(self, context, final):
+        handled_lhs = self.__lhs.handle_ss3(context, final)
+        handled_rhs = self.__rhs.handle_ss3(context, final)
+        return handled_lhs and handled_rhs
+
     def handle_control_string(self, context, prefix, value):
         handled_lhs = self.__lhs.handle_control_string(context, prefix, value)
         handled_rhs = self.__rhs.handle_control_string(context, prefix, value)
@@ -897,7 +925,7 @@ class ParseContext(OutputStream, EventDispatcher):
 #
 class DefaultPTY(PTY):
 
-    def __init__(self, term, lang, command, stdin):
+    def __init__(self, term, lang, command, stdin, row=None, col=None):
         self._stdin_fileno = stdin.fileno()
         backup = termios.tcgetattr(self._stdin_fileno)
         self._backup_termios = backup
@@ -905,14 +933,15 @@ class DefaultPTY(PTY):
         if not pid:
             os.environ['TERM'] = term
             os.environ['LANG'] = lang
-            os.execlp('/bin/sh',
-                      '/bin/sh', '-c',
-                      'exec %s' % command)
+            os.execlp('/bin/sh', '/bin/sh', '-c', 'exec %s' % command)
 
         self.__setupterm(self._stdin_fileno)
         self.pid = pid
         self._master = master
 
+        if row and col:
+            self.resize(row, col)
+
     def close(self):
         #self.restore_term()
         try:
@@ -1017,6 +1046,7 @@ class DefaultPTY(PTY):
         #fcntl.ioctl(self._master, termios.TIOCSTART, 0)
         termios.tcflow(self._master, termios.TCOON)
 
+
 class MockParseContext(ParseContext):
 
     def __init__(self):
@@ -1071,7 +1101,7 @@ class Process:
         return self._tty.pid
 
     def is_alive(self):
-        return self._tty is not None 
+        return self._tty is not None
 
     def fileno(self):
         return self._tty.fileno()
@@ -1086,8 +1116,9 @@ class Process:
         self._tty.write(data)
 
     def close(self):
-        if self._tty is not None:
-            self._tty.close()
+        tty = self._tty
+        if tty is not None:
+            tty.close()
             self._tty = None
 
     def end(self):
@@ -1179,23 +1210,35 @@ class Session:
         self._resized = False
         self._process_map = {}
 
+    # deprecated
     def add_subtty(self, term, lang,
                    command, row, col,
                    termenc,
                    inputhandler=DefaultHandler(),
                    outputhandler=DefaultHandler()):
+        return self.create_process(term, lang, command, row, col,
+                                   termenc, inputhandler, outputhandler)
 
-        tty = DefaultPTY(term, lang, command, sys.stdin)
-        tty.resize(row, col)
+    def create_process(self, term, lang,
+                       command, row, col,
+                       termenc,
+                       inputhandler=DefaultHandler(),
+                       outputhandler=DefaultHandler(),
+                       inputparser=DefaultParser(),
+                       outputparser=DefaultParser(),
+                       inputscanner=DefaultScanner(),
+                       outputscanner=DefaultScanner(),
+                       buffering=False):
+
+        tty = DefaultPTY(term, lang, command, sys.stdin, row, col)
         process = Process(tty)
 
         self._init_process(process,
                            termenc,
                            inputhandler, outputhandler,
-                           DefaultParser(), DefaultParser(),
-                           DefaultScanner(), DefaultScanner(),
-                           buffering=False)
-        self.focus_process(process)
+                           inputparser, outputparser,
+                           inputscanner, outputscanner,
+                           buffering=buffering)
         return process
 
     def getactiveprocess(self):
@@ -1213,7 +1256,8 @@ class Session:
     def blur_process(self):
         process = self._mainprocess
         if process.is_alive():
-            logging.info("Switching focus: fileno=%d (main process)" % process.fileno())
+            template = "Switching focus: fileno=%d (main process)"
+            logging.info(template % process.fileno())
             self._input_target.drain()
             self._input_target = process
 
@@ -1331,7 +1375,6 @@ class Session:
                            inputparser, outputparser,
                            inputscanner, outputscanner,
                            buffering)
-        self.focus_process(mainprocess)
 
         self._resized = False
 
@@ -1369,6 +1412,7 @@ class Session:
                       inputscanner,
                       outputscanner,
                       buffering)
+        self.focus_process(process)
 
 
 def _test():
@@ -1381,7 +1425,7 @@ if __name__ == '__main__':
     import inspect
     import hashlib
     import sys
-    
+
     thismodule = sys.modules[__name__]
     md5 = hashlib.md5()
     specs = []
@@ -1399,5 +1443,3 @@ if __name__ == '__main__':
     specs.sort()
     md5.update("".join(specs))
     sys.stdout.write(md5.hexdigest())
-
-
