@@ -24,7 +24,7 @@ import re
 import settings
 import logging
 import time
-import socket
+import thread
 
 homedir = os.path.expanduser('~')
 rcdir = os.path.join(homedir, '.sskk')
@@ -48,6 +48,7 @@ _okuridb = {}
 
 user_dict_file = None
 
+
 class SkkLineLoader():
 
     _encoding = 0
@@ -57,14 +58,17 @@ class SkkLineLoader():
     def _decode_line(self, line):
         try:
             return unicode(line, self._encoding_list[self._encoding])
-        except UnicodeDecodeError, e:
-            self._encoding = 1 - self._encoding # flip
+        except UnicodeDecodeError:
+            self._encoding = 1 - self._encoding  # flip
         return unicode(line, self._encoding_list[self._encoding])
 
     def parse(self, line):
         if len(line) < 4 or line[1] == ';':
             return
-        line = self._decode_line(line)
+        try:
+            line = self._decode_line(line)
+        except UnicodeDecodeError:
+            return
         match = self._pattern.match(line)
         if not match:
             template = '_load_dict: can\'t load the entry: %s'
@@ -95,8 +99,6 @@ class SkkLineLoader():
                 _tangodb[alphakey] = value.split('/')
 
 
-
-
 class Expander():
 
     limit = 10
@@ -110,7 +112,7 @@ class Expander():
                 candidate.append(key + c)
             else:
                 self._expand_all(key + c, (x for x in value.items()), candidate)
-    
+
     def _expand_sparse(self, key, current, candidate, generators):
         limit = self.limit
         for c, value in current:
@@ -124,7 +126,7 @@ class Expander():
                 generators.append((key, current))
                 return True
         return False
-    
+
     def expand(self, key, current, candidate):
         limit = settings.get('suggest.max')
         self.limit = limit
@@ -149,6 +151,7 @@ class Expander():
                 elif not current:
                     candidate.append(key)
 
+
 class Completer():
 
     def __init__(self):
@@ -174,9 +177,9 @@ class Completer():
                 _current = _current[_c]
             else:
                 return None
-    
+
         expander = self._expander
-    
+
         candidate = list()
         if not finals:
             expander.expand(u'', _current, candidate)
@@ -202,6 +205,7 @@ completer = Completer()
 
 _control_chars = re.compile('[\x00-\x1f\x7f\x80-\x9f\xff]')
 
+
 def feedback(key, value):
     global user_dict_file
     if not key:
@@ -239,9 +243,10 @@ def _escape(s):
 def _load_dict(filename):
     try:
         thread.start_new_thread(_load_dict_impl, (filename))
-    except Exception, e:
+    except Exception:
         _load_dict_impl(filename)
 
+
 def _load_dict_impl(filename):
 
     loader = SkkLineLoader()
@@ -265,7 +270,7 @@ def _get_fallback_dict_path(name):
 def _load_history(filename):
     try:
         thread.start_new_thread(_load_history_impl, (filename))
-    except Exception, e:
+    except Exception:
         _load_history_impl(filename)
 
 
@@ -287,6 +292,7 @@ def _load_history_impl(filename):
         else:
             _tangodb[key] = [value]
 
+
 def _load_user_dict():
     """
     >>> _load_user_dict()
@@ -337,6 +343,7 @@ def _load_user_dict():
         except ImportError, e:
             logging.exception(e)
 
+
 def _load_builtin_dict():
     """
     >>> _load_user_dict()
@@ -398,6 +405,7 @@ def _load_builtin_dict():
 
     logging.info("Dictionary initialization processes are completed.")
 
+
 def gettango(key):
     if key in _user_tangodb:
         result = _user_tangodb[key]
@@ -484,7 +492,7 @@ def get_from_cgi_api(clauses, key):
             key, cgi_candidates = clauseinfo
             candidates = gettango(key)
             for candidate in cgi_candidates:
-                if not candidate in candidates: 
+                if not candidate in candidates:
                     candidates.append(candidate)
             clause = Clause(key, candidates)
             clauses.add(clause)
@@ -634,6 +642,7 @@ class Clauses:
 
 
 def _create_dns_cache():
+    import socket
     """
     create DNS cache for www.google.com
     """
@@ -644,17 +653,18 @@ def _create_dns_cache():
         logging.warning(e)
 
 
-# load dictionaries asynchronously if possible
-try:
-    import thread
-    thread.start_new_thread(_create_dns_cache, ())
-    thread.start_new_thread(_load_user_dict, ())
-    thread.start_new_thread(_load_builtin_dict, ())
-except Exception, e:
-    logging.warning(e)
-    logging.warning("Fallback to synchronous initialization for dictionaries.")
-    _load_user_dict()
-    _load_builtin_dict()
+def initialize():
+    # load dictionaries asynchronously if possible
+    try:
+        thread.start_new_thread(_create_dns_cache, ())
+        thread.start_new_thread(_load_user_dict, ())
+        thread.start_new_thread(_load_builtin_dict, ())
+    except Exception, e:
+        logging.warning(e)
+        logging.warning("Fallback to synchronous initialization for dictionaries.")
+        _load_user_dict()
+        _load_builtin_dict()
+
 
 def test():
     import doctest
